# import from python libraries
import os
from attrdict import AttrDict

try:
    import magic
    MAGIC_IMPORTED = True
except ImportError:
    MAGIC_IMPORTED = False

try:
    import mimetypes
    MIMETYPES_IMPORTED = True
except ImportError:
    MIMETYPES_IMPORTED = False

def file_access_check(filepath, mode=os.R_OK, raise_exception=False):
    """ helper method to check file access"""
    if not filepath:
        raise AttributeError('missing file error')

    if (not os.path.exists(filepath) or
        not os.path.isfile(filepath) or
        not os.access(filepath, mode)):
            if raise_exception:
                raise AttributeError("'%' file access error" % filepath)
            return False

    return True

def reverse_attrdict(dict_data):
    """ helper method to recursivly convert dictionary to attrdict properties"""
    if not isinstance(dict_data, dict):
        return dict_data

    for key in list(dict_data.keys()):
        value = reverse_attrdict(dict_data.pop(key))
        key = key.replace('-', '_')
        dict_data[key] = AttrDict(value) if isinstance(value, dict) else value

    return dict_data

def deseralize(cls, input):
    """ helper method to deseralize json payload Veem Object"""
    return cls(**input) if isinstance(input, dict) else input

def extract_file_content_type(filename, buffer=None):
    """ helper method to extract file content type"""
    try:
        if MIMETYPES_IMPORTED:
            mimetype, encoding = mimetypes.guess_type(filename)
            return mimetype
        if mimetype is None and MAGIC_IMPORTED:
            if buffer:
                return magic.from_buffer(buffer, mime=True)
            return magic.from_file(filename, mime=True)
    except Exception:
        pass
    return "multipart/form-data"




from veem.models.base import Base

class Amount(Base):
    def __init__(self, number=None, currency=None, **kwargs):

        self._validate_currency_code(currency)
        self.number = number
        self._currency = currency

    @property
    def currency(self):
        return self._currency

    @currency.setter
    def currency(self, value):
        self._validate_currency_code(value)
        self._currency = value




from veem.models.base import Base
from veem.models.batch_item import BatchItem

from veem.utils import deseralize
from veem.constants import BatchStatus

class Batch(Base):
    def __init__(self,
                 id=None,
                 batchId=None,
                 status=None,
                 hasErrors=None,
                 processedItems=None,
                 totalItems=None,
                 batchItems=[],
                 **kwargs):

        self._validate_constants(BatchStatus, status)

        self.id = id or batchId
        self.batchId = id or batchId
        self.status = status
        self.hasErrors = hasErrors
        self.processedItems = processedItems
        self.totalItems = totalItems
        self.batchItems = [deseralize(BatchItem, b) for b in batchItems]




class Base(object):

    @property
    def json(self):
        return {k:v for k,v in self.__dict__.items() if not k.startswith('_')}

    @property
    def convert(self):
        return self

    def _validate_constants(self, constant, value, required=False):
        if not value and not required: return
        if isinstance(value, list):
            for v in value:
                self._validate_constants(constant, v)
            return
        if isinstance(constant, tuple) and isinstance(value, dict):
            k_constant, v_constant = constant
            for k,v in value.items():
                self._validate_constants(k_constant, k)
                self._validate_constants(k_constant, v)
            return
        if not value in constant.values():
            raise ValueError('Unsupport Constant Value %s' % value)

    def _validate_country_code(self, value, required=False):
        if not value and not required: return
        if len(str(value)) != 2:
            raise ValueError('Unsupport CountryCode %s' % value)

    def _validate_currency_code(self, value, required=False):
        if not value and not required: return
        if len(str(value)) != 3:
            raise ValueError('Unsupport CurrencyCode %s' % value)




from veem.models.base import Base

from veem.utils import deseralize
from veem.exceptions import VeemSdkException
from veem.constants import BatchItemType, BatchStatus

class ErrorInfo(Base):

    def __init__(self,
                 code=None,
                 error=None,
                 message=None,
                 timestamp=None,
                 **kwargs):

        self.code = code
        self.error = error
        self.message = message
        self.timestamp = timestamp


class BatchItem(Base):
    def __init__(self,
                 id=None,
                 batchItemId=None,
                 status=None,
                 errorInfo=None,
                 paymentId=None,
                 contactId=None,
                 type=None,
                 **kwargs):

        self._validate_constants(BatchStatus, status)
        self._validate_constants(BatchItemType, type)

        self.id = id or batchItemId
        self.batchItemId = id or batchItemId
        self.status = status
        self.errorInfo = errorInfo
        self._paymentId = paymentId
        self._contactId = contactId
        self.type = type

    @property
    def paymentId(self):
        if self.type == BatchItemType.PAYMENT:
            return self._paymentId
        raise VeemSdkException(("Can't get paymentId from BatchItem with "
                                "type {}").format(self.type));

    @property
    def contactId(self):
        if self.type == BatchItemType.CONTACT:
            return self._contactId
        raise VeemSdkException(("Can't get contactId from BatchItem with "
                                "type {}").format(self.type));



from veem.client.responses.page import PageResponse
from veem.client.responses.account import AccountResponse

from veem.client.base import Base
from veem.utils.rest import VeemRestApi

class CustomerClient(Base):

    def __init__(self, config, **kwargs):

        self.config = config
        self.context = config.context
        self.client = VeemRestApi(self.config.url,
                                  self.context.session,
                                  dict(list=('get', '')))

    def list(self, request):
        """
            Matching Veem customer email with provided email address

            @param request: an AccountRequest or stirng of email
            @return paginated Accounts that matches the search criteria.
        """
        email = getattr(request, 'email', str(request))
        return self._list_response_handler(PageResponse,
                                 AccountResponse,
                                 self.client.list(
                                            request_params=dict(email=email),
                                            access_token=self.context.token,
                                            api_route='customers')
                                        )




from veem.models.base import Base

from veem.constants import AttachmentType

class Attachment(Base):
    def __init__(self,
                 type=None,
                 name=None,
                 referenceId=None,
                 path=None,
                 **kwargs):

        self.type = type
        self.name = name
        self.referenceId = referenceId
        self.path = path

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, value):
        self._validate_constants(AttachmentType, value)
        self._type = value



#! /bin/env python

'''Setup file for Veem Python SDK

See:
    https://packaging.python.org/en/latest/distributing.html
'''
import os
import re
from setuptools import find_packages, setup

def read(*file):
    with open(os.path.join(os.path.abspath(os.path.dirname(__file__)),
                           *file)) as f:
        content = f.read()
    return content

def version_info(*args):
    content = read(*args)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              content, re.M)
    if version_match:
        return version_match.group(1)
    return '1.0.0'

# get version information
version = version_info('src', 'veem', '__init__.py')

# launch setup
setup(
    name = 'oveem',
    version = version,

    # descriptions
    description = ('Veem Python SDK provides an interface to call '
                   'Veem Global Payments APIs (https://www.veem.com).'),
    long_description = read('DESCRIPTION.md'),
    long_description_content_type='text/markdown',
    author = 'Intelligenti',
    author_email = 'hello@intelligenti.io',
    url = 'https://github.com/intelligenti/veem-python-sdk',

    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Financial and Insurance Industry',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Office/Business :: Financial',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    # project keywords
    keywords = 'veem python intelligenti sdk b2b payment',

    # uses namespace package
    namespace_packages = [],

    # project packages
    packages = find_packages(where = 'src'),

    # project directory
    package_dir = {
        '': 'src',
    },

    # additional package data files that goes into the package itself
    package_data = {
        '': ['README.md']
    },

    # console entry point
    entry_points = {
    },

    # package dependencies
    install_requires = [
        'six',
        'pyyaml',
        'urllib3',
        'attrdict',
        'requests',
    ],

    # any additional groups of dependencies.
    # install using: $ pip install -e .[dev]
    extras_require = {
    },

    # external modules
    ext_modules = [],

    # unit test suite
    test_suite="",

    # any data files placed outside this package.
    # See: http://docs.python.org/3.4/distutils/setupscript.html
    # format:
    #   [('target', ['list', 'of', 'files'])]
    # where target is sys.prefix/<target>
    data_files = [],

    zip_safe = False,
)



import urllib.parse

from veem.constants import (
    Scope,
    CLIENT_CREDENTIALS_GRANT_TYPE,
    AUTHORIZATION_CODE_GRANT_TYPE,
    REFRESH_TOKEN_GRANT_TYPE
)
from veem.client.responses.token import TokenResponse

from veem.client.base import Base
from veem.utils.rest import VeemRestApi

class AuthenticationClient(Base):

    def __init__(self, config, **kwargs):

        self.config = config
        self.context = config.context
        self.client = VeemRestApi(self.config.url,
                                  self.context.session,
                                  dict(getTokenFromClientCredentials=(
                                        'post', 'oauth/token'),
                                       refreshToken=('post', 'oauth/token'),
                                       getTokenFromAuthorizationCode=(
                                       'post', 'oauth/token')))

    def getTokenFromClientCredentials(self, scope=Scope.All):
        """
            Get Access token using API Client Credential
        """
        response = self._response_handler(TokenResponse,
            self.client.getTokenFromClientCredentials(
            request_auth=(self.context.clientId, self.context.clientSecret),
            request_params=dict(grant_type=CLIENT_CREDENTIALS_GRANT_TYPE,
                                scope=scope)
        ))
        self.context.token = response
        return response

    def refreshToken(self, refreshToken):
        """
            Refresh Access token
        """
        response = self._response_handler(TokenResponse,
            self.client.refreshToken(
            request_auth=(self.context.clientId, self.context.clientSecret),
            request_params=dict(grant_type=REFRESH_TOKEN_GRANT_TYPE,
                                refresh_token=refreshToken)
        ))
        self.context.token = response
        return response

    def getTokenFromAuthorizationCode(self, scope=Scope.All):
        """
            Refresh Access token from authorization code
        """
        print(self.self.context.redirectUrl, urllib.parse.quote(self.context.redirectUrl), 'HERE, HERE HERE')
        response = self.client.refreshToken(
            request_auth=(self.context.clientId, self.context.clientSecret),
            request_params=dict(grant_type=AUTHORIZATION_CODE_GRANT_TYPE,
                           code=self.context.authorizationCode,
                           redirect_uri=urllib.parse.quote(self.context.redirectUrl),
                           scope=scope))
        return response



from veem.models.base import Base
from veem.models.country_info import CountryInfo
from veem.models.purpose_of_payment import PurposeOfPayment

from veem.utils import deseralize

class CountryInfoResponse(Base):
    def __init__(self,
                 bankFields=None,
                 country=None,
                 invoiceAttachmentRequired=None,
                 purposeOfPaymentInfo=[],
                 purposeOfPaymentRequired=None,
                 receivingCurrencies=[],
                 sendingCurrencies=[],
                 **kwargs):

        self.bankFields = bankFields
        self.country = country
        self.purposeOfPaymentInfo = [deseralize(PurposeOfPayment,
                                       info) for info in purposeOfPaymentInfo]
        self.purposeOfPaymentRequired = purposeOfPaymentRequired
        self.invoiceAttachmentRequired = invoiceAttachmentRequired
        self.receivingCurrencies = receivingCurrencies
        self.sendingCurrencies = sendingCurrencies

    @property
    def convert(self):
        return CountryInfo(**self.json)
